/**
 * @file app_dm.c
 * @author your name (you@domain.com)
 * @brief
 * @version 0.1
 * @date 2023-09-22
 *
 * @copyright Copyright (c) 2023
 *
 */

/* Includes --------------------------------------------------------------------------------------*/
#include "app_dm.h"
#include "ble_services_peripheral.h"

/* Global param ----------------------------------------------------------------------------------*/
K_TIMER_DEFINE(dm_timer, NULL, NULL);

/* Private function prototype --------------------------------------------------------------------*/
static void dm_timer_callback (struct k_timer *p_timer);
void print_result(struct dm_result *result);

/**
 * @brief Timer timeout callback function for DM
 *
 * @param p_timer
 */
static void dm_timer_callback (struct k_timer *p_timer)
{
	app_task_msg_t msg;
   	msg.event = EVENT_PERIODIC_DM_REQ;
	k_msgq_put(&app_msgq, &msg, K_NO_WAIT);
}

/**
 * @brief Print DM result
 *
 * @param result
 */
void print_result (struct dm_result *result)
{
	if (!result) {
		return;
	}

	const char *quality[DM_QUALITY_NONE + 1] = {"Ok", "Poor", "Do not use", "CRC fail", "None"};
	printk("\tQuality: %s\n", quality[result->quality]);
	printk("\tDistance estimates: mcpd: best = %.2f\n", (double)result->dist_estimates.mcpd.best);
}

/**
 * @brief Add DM request for beacon device
 *
 * @param seed
 * @param addr
 */
void beacon_dm_add_request (uint32_t seed, bt_addr_le_t addr)
{
    struct dm_request req;

	bt_addr_le_copy(&req.bt_addr, &addr);
	req.role = DM_ROLE_REFLECTOR;
	req.ranging_mode = DM_RANGING_MODE_MCPD;
	req.rng_seed = seed;
	req.start_delay_us = 0;
	req.extra_window_time_us = 0;
	dm_request_add(&req);
}

/**
 * @brief Add DM request for detection device
 *
 * @param seed
 * @param addr
 */
void detection_dm_add_request (uint32_t seed, bt_addr_le_t addr)
{
    struct dm_request req;

	bt_addr_le_copy(&req.bt_addr, &addr);
	req.role = DM_ROLE_INITIATOR;
	req.ranging_mode = DM_RANGING_MODE_MCPD;
	req.rng_seed = seed;
	req.start_delay_us = 0;
	req.extra_window_time_us = 0;
	dm_request_add(&req);
}

/**
 * @brief Callback function, called when DM data is ready
 *
 * @param result
 */
void dm_data_ready (struct dm_result *result)
{
	if (!result) {
		return;
	}
	static app_task_msg_t msg;
   	msg.event = EVENT_DM_RESULT_READY;
	msg.data = result;
	memcpy(&msg.result, result, sizeof(struct dm_result));
	k_msgq_put(&app_msgq, &msg, K_NO_WAIT);
}

struct dm_cb dm_cb = {
	.data_ready = dm_data_ready,
};

/**
 * @brief Initialize Distance Measurement module for DM of the peer devices
 * Also configure a timer for periodic DM
 *
 */
int app_dm_init (void)
{
    int err;

	struct dm_init_param init_param;

	init_param.cb = &dm_cb;
	err = dm_init(&init_param);
	if (err) {
		return err;
	}

	k_timer_init(&dm_timer, dm_timer_callback, NULL);

    return 0;
}
