/**
 * @file app_nvs.c
 * @author Adnan
 * @brief
 * @version 0.1
 * @date 2023-05-26
 *
 * @copyright Copyright (c) 2023
 *
 */
/* Includes --------------------------------------------------------------------------------------*/
#include "app_nvs.h"

/* Macros ----------------------------------------------------------------------------------------*/
#define NVS_PARTITION			storage_partition
#define NVS_PARTITION_DEVICE	FIXED_PARTITION_DEVICE(NVS_PARTITION)
#define NVS_PARTITION_OFFSET	FIXED_PARTITION_OFFSET(NVS_PARTITION)
#define NVS_DATA_ID 			(1)

#define NVS_SECTORS_COUNT       (2)

/* Private param ---------------------------------------------------------------------------------*/
static struct nvs_fs fs;

int rc = 0;
struct flash_pages_info info;

app_nvs_t app_nvs;

/**
 * @brief Read requested length of data from the NVS partition
 *
 * @param p_data - return pointer to data buffer
 * @param len    - length of data to read from the NVS partition
 * @return true  - successfully read from the NVS
 * @return false - failed to read from the NVS
 */
bool read_from_nvs (void *p_data, int len)
{
    rc = nvs_read(&fs, NVS_DATA_ID, p_data, len);
    return (rc > 0);
}

/**
 * @brief Write the requested length of data from the NVS partition
 *
 * @param p_data - pointer to data to be written onto the NVS
 * @param len    - length of data to write to the NVS partition
 * @return true  - successfully written to the NVS
 * @return false - failed to write to the NVS
 */
bool write_to_nvs (void *p_data, int len)
{
    rc = nvs_write(&fs, NVS_DATA_ID, p_data, len);
	return (rc >= 0);
}

/**
 * @brief Update our data in the NVS
 * Retry few times on failure and exit
 *
 * @return true  - Data is successfully written to the NVS
 * @return false - Data is not written to NVS
 */
bool update_nvs (void)
{
    app_nvs_t rd_nvs_data;

    for (uint8_t retry = 0; retry < 3; retry++) {
        if (write_to_nvs(&app_nvs, sizeof(app_nvs)) &&
            read_from_nvs(&rd_nvs_data, sizeof(rd_nvs_data)) &&
            memcmp(&rd_nvs_data, &app_nvs, sizeof(app_nvs)) == 0)
        {
            return true;
        }
    }

    return false;
}

/**
 * @brief Update paired device connection info, device ID in the NVS
 *
 * @param p_id: pointer to the device unique ID
 */
void update_nvs_connection_param (uint8_t *p_id)
{
	for (uint8_t dev = 0; dev < app_nvs.param.paired_device_count; dev++) {
		if (!memcmp(&app_nvs.param.pair_info[dev].id, p_id, BT_ADDR_SIZE) &&
            (paired_list[dev].pair_status == 1)) {
			return;
		}
	}

	memcpy(app_nvs.param.pair_info[app_nvs.param.paired_device_count].id, p_id, BT_ADDR_SIZE);
	app_nvs.param.paired_device_count++;
	update_nvs();
}

/**
 * @brief Print all the information stored in the NVS
 *
 */
void print_nvs_info (void)
{
	printk("\nNVS info:\n");

	printk("Device mode: %s\n", app_nvs.param.mode ? "Detection" : "Beacon");
	printk("Num of device paired: %d\n", app_nvs.param.paired_device_count);
	for (uint8_t dev = 0; dev < app_nvs.param.paired_device_count; dev++) {
		printk("\n%d: %02x : %02x : %02x : %02x : %02x : %02x\n", dev + 1,
				app_nvs.param.pair_info[dev].id[0], app_nvs.param.pair_info[dev].id[1], app_nvs.param.pair_info[dev].id[2],
				app_nvs.param.pair_info[dev].id[3], app_nvs.param.pair_info[dev].id[4], app_nvs.param.pair_info[dev].id[5]);
	}
}

/**
 * @brief Initialize the NVS partition
 *
 */
bool nvs_init (void)
{
	/* Define the nvs file system by settings with:
	 *	sector_size equal to the pagesize,
	 *	2 sectors
	 *	starting at NVS_PARTITION_OFFSET
	 */
	fs.flash_device = NVS_PARTITION_DEVICE;
	if (!device_is_ready(fs.flash_device)) {
		return false;
	}
	fs.offset = NVS_PARTITION_OFFSET;
	rc = flash_get_page_info_by_offs(fs.flash_device, fs.offset, &info);
	if (rc) {
		return false;
	}
	fs.sector_size = info.size;
	fs.sector_count = NVS_SECTORS_COUNT;

	rc = nvs_mount(&fs);
	if (rc) {
		return false;
	}

	/* NVS_DATA_ID is used to store the list of connected devices, lets see if we can read it from
       flash */
	rc = read_from_nvs(&app_nvs, sizeof(app_nvs));
	if (!rc) {
        // Clear NVS structure, set default values and write it to the NVS partition as fresh data
        //
        memset(&app_nvs, 0, sizeof(app_nvs));
        app_nvs.param.zone_range = ZONE_1;
		rc = write_to_nvs(&app_nvs, sizeof(app_nvs));
		if (rc <= 0) {
			return false;
		}
	}

    return true;
}
