/* main.c - Application main entry point */

/*
 * Copyright (c) 2021 Nordic Semiconductor ASA
 * Copyright (c) 2015-2016 Intel Corporation
 *
 * SPDX-License-Identifier: Apache-2.0
 */

/* Includes --------------------------------------------------------------------------------------*/
#include "app_common.h"
#include <bluetooth/scan.h>
#include <zephyr/bluetooth/hci.h>
#include <zephyr/bluetooth/conn.h>
#include <zephyr/bluetooth/uuid.h>
#include <zephyr/sys/byteorder.h>
#include "ble_services_peripheral.h"

/* Private macros --------------------------------------------------------------------------------*/
#define SCAN_INTERVAL       (0x0640) // 1000 ms (N * 0.625ms)
#define SCAN_WINDOW         (0x00A0) // 100 ms (N * 0.625ms)

/* Private param ---------------------------------------------------------------------------------*/
static struct bt_uuid_128 g_peer_device_uuid = BT_UUID_INIT_128(0);
static struct bt_gatt_discover_params g_discover_params;
static struct bt_gatt_subscribe_params g_subscribe_params;
volatile bool gb_scanning_in_progress = false;
volatile bool gb_dm_write_resp_wait = false;

/* Global param ----------------------------------------------------------------------------------*/
int8_t g_current_index = 0;
int8_t g_dm_index = 0;
paired_dev_info_t paired_list[MAX_BEACONS_SUPPORTED];
paired_dev_info_t current_dev_info;
uint32_t g_mobile_id = 0;
bt_addr_le_t *gp_dissociate_addr;

/* Private function prototypes -------------------------------------------------------------------*/
static void beacon_connected (struct bt_conn *conn, uint8_t reason);
static void beacon_disconnected (struct bt_conn *conn, uint8_t reason);
static bool le_param_req (struct bt_conn *conn, struct bt_le_conn_param *param);
static uint8_t discover_cb (struct bt_conn *conn, const struct bt_gatt_attr *attr,
			                struct bt_gatt_discover_params *params);
static bool eir_found (struct bt_data *data, void *user_data);
static void device_found (const bt_addr_le_t *addr, int8_t rssi, uint8_t type,
			 			  struct net_buf_simple *ad);
/**
 * @brief Callback function, called when any beacon device is connected
 *
 * @param conn
 * @param reason
 */
static void beacon_connected (struct bt_conn *conn, uint8_t reason)
{
	char addr_str[BT_ADDR_LE_STR_LEN];

	if (reason) {
		bt_conn_unref(conn);
		start_scan();
		return;
	}

	// New BLE connection established
	//
	current_dev_info.conn = conn;
	current_dev_info.conn_status = CONN_STATUS_CONNECTING;

	discover_services();

	bt_addr_le_to_str(bt_conn_get_dst(conn), addr_str, sizeof(addr_str));
}

/**
 * @brief Callback function, called when any beacon device is disconnected
 *
 * @param conn
 * @param reason
 */
static void beacon_disconnected (struct bt_conn *conn, uint8_t reason)
{
	static app_task_msg_t msg;
	char addr_str[BT_ADDR_LE_STR_LEN];

	bt_addr_le_to_str(bt_conn_get_dst(conn), addr_str, sizeof(addr_str));

	// Check if the device is paired or new.
	// If paired device, set as disconnected and alert the user
	// If new device, ignore
	//
	int8_t index = get_paired_device_slot_from_conn_param(conn);
	if (index >= 0) {
		if (paired_list[index].conn) {
			bt_conn_unref(conn);
		}
		paired_list[index].conn = NULL;
		paired_list[index].conn_status = CONN_STATUS_DISCONNECTED;

        // Check for alert only if we are at zone ranging, we disconnect from all the devices when
        // user request factory reset or mode transit so not necessary for alerting peripheral at
        // this state
        //
        if (paired_list[index].pair_status &&
            (dev_info.state != APP_STATE_MODE_TRANSIT) &&
            (dev_info.state != APP_STATE_FACTORY_RESET))
        {
    		if (dev_info.ack ? 1 :
                (!dev_info.alert_status && !paired_list[index].zone_breach_status &&
                 !dev_info.silent_mode))
            {
    			dev_info.ack = false;
    			if (!paired_list[index].zone_breach_status) {
    				paired_list[index].zone_breach_status = true;
    			}

    			msg.event = EVENT_UPDATE_STATE;
    			msg.state = APP_STATE_ALERT_START;
    			k_msgq_put(&app_msgq, &msg, K_NO_WAIT);
    		}
        }
	}
    else if (reason != BT_HCI_ERR_LOCALHOST_TERM_CONN) {
        // Unknown or unpair device disconnected, remove reference from the BLE stack
        // NOTE: If the peer disconnected by the local host, no need to unref the connection as it
        //       is already handled.
        //
		bt_conn_unref(conn);
    }

    if ((dev_info.state != APP_STATE_MODE_TRANSIT) && (dev_info.state != APP_STATE_FACTORY_RESET)) {
        // Quit from APP_STATE_WAIT_CONFIRM
        //
        if (app_nvs.param.paired_device_count) {
            // We have one or more device paired, check for breach alert before switching app state
            // If we are alerting, blink RED led
            // Else, switch to zone ranging
            //
            if (dev_info.alert_status) {
                led_fast_blink(LED_COLOR_RED);
            }
            else {
                led_off(); // turn OFF GREEN LED
            }
        }
        else {
            led_off(); // no device is being monitored, turn OFF GREEN LED
        }

        start_scan();
    }
}

/**
 * @brief BT connection param request callback function
 *
 * @param conn
 * @param param
 * @return true
 * @return false
 */
static bool le_param_req (struct bt_conn *conn, struct bt_le_conn_param *param)
{
    return true;
}

static struct bt_conn_cb conn_callbacks = {
	.connected    = beacon_connected,
	.disconnected = beacon_disconnected,
	.le_param_req = le_param_req,
};

/**
 * @brief Callback function that is called when a connected peripheral sends a indication on the
 * cmd char
 *
 *  @param conn Connection object. May be NULL, indicating that the peer is
 *              being unpaired
 *  @param params Subscription parameters.
 *  @param data Attribute value data. If NULL then subscription was removed.
 *  @param length Attribute value length.
 * @return uint8_t BT_GATT_ITER_CONTINUE to continue receiving value notifications.
 *          	   BT_GATT_ITER_STOP to unsubscribe from value notifications.
 */
uint8_t indication_cb (struct bt_conn *conn, struct bt_gatt_subscribe_params *params,
			           const void *data, uint16_t length)
{
	static int8_t device_idx = -1;
    device_idx = get_paired_device_slot_from_conn_param(conn);

	static app_task_msg_t msg;
	char addr_str[BT_ADDR_LE_STR_LEN];
	bt_addr_le_to_str(bt_conn_get_dst(conn), addr_str, sizeof(addr_str));
	uint8_t opcode = ((uint8_t *)data)[BYTE_OFF_OPCODE];

	if (data && device_idx >= 0) {
    	if (dev_info.state >= APP_STATE_WAIT_CONFIRM) {
    		if (opcode == OPCODE_CONFIRM_PARING) {
               	msg.event = EVENT_UPDATE_STATE;
            	msg.state = APP_STATE_PAIRED;
    		}
			else if (opcode == OPCODE_STOP_ALERT) {
				msg.event = EVENT_UPDATE_STATE;
				msg.state = APP_STATE_ALERT_STOP;
			}
			else if (opcode == OPCODE_SET_ZONE) {
				msg.event = EVENT_UPDATE_STATE;
				msg.state = APP_STATE_ZONE_SETUP;
				if (((uint8_t *)data)[BYTE_OFF_DATA] >= ZONE_1 &&
                    ((uint8_t *)data)[BYTE_OFF_DATA] <= ZONE_SILENT)
                {
				    dev_info.zone_range = ((uint8_t *)data)[BYTE_OFF_DATA];

                    // If the zone is set from the mobile app, calibrate the zone values as below
                    // 7: ZONE_1, 12: ZONE_2, 20: ZONE_3, 25: ZONE_SILENT
                    //
                    switch (dev_info.zone_range) {
                        case 7:
                            dev_info.zone_range = ZONE_1;
                            break;

                        case 12:
                            dev_info.zone_range = ZONE_2;
                            break;

                        case 20:
                            dev_info.zone_range = ZONE_3;
                            break;

                        default:
                            break;
				    }

					turn_on_zone_indicator_led();
                }
			}
            else if (opcode == OPCODE_DISSOCIATE) {
                gp_dissociate_addr = (bt_addr_le_t *)bt_conn_get_dst(conn);
                msg.event = EVENT_DISSOCIATE;
				msg.data = &device_idx;
            }
            else if (opcode == OPCODE_OTA_FOR_DETECTION_DEVICE) {
                msg.event = EVENT_UPDATE_STATE;
                msg.state = APP_STATE_SETUP_FOR_OTA;
            }
            else if (opcode == OPCODE_OTA_FOR_BEACON_DEVICE) {
                msg.event = EVENT_UPDATE_STATE;
                msg.state = APP_STATE_SETUP_BEACON_FOR_OTA;
            }
            else if (opcode == OPCODE_FW_VER_INFO) {
                paired_list[device_idx].fw_version_major = ((uint8_t *)data)[BYTE_OFF_DATA];
                paired_list[device_idx].fw_version_minor = ((uint8_t *)data)[BYTE_OFF_DATA + 1];

                // Nothing to process in main thread
                //
                msg.event = EVENT_NONE;
            }
			else if (opcode == OPCODE_SET_SILENT_MODE) {
				dev_info.silent_mode = ((uint8_t *)data)[BYTE_OFF_DATA];
				msg.event = EVENT_UPDATE_STATE;
                msg.state = APP_STATE_SILENT_MODE;
				turn_on_zone_indicator_led();
			}
			else if (opcode == OPCODE_DISABLE_SILENT_MODE) {
				dev_info.silent_mode = ((uint8_t *)data)[BYTE_OFF_DATA];
				msg.event = EVENT_UPDATE_STATE;
                msg.state = APP_STATE_DISABLE_SILENT_MODE;
			}
			else if (opcode == OPCODE_GET_MOBILE_ID) {
				uint8_t *pu8_data = (uint8_t *)data;
				memcpy(current_dev_info.id, &pu8_data[BYTE_OFF_DATA], sizeof(current_dev_info.id));

                msg.event = EVENT_UPDATE_STATE;
			    msg.state = APP_STATE_CHECK_EXISTENCE;
			}
			else if (opcode == OPCODE_FIND_DEVICE) {
				msg.event = EVENT_UPDATE_STATE;
				msg.state = APP_STATE_FIND_THE_DEVICE;
			}
			else if (opcode == OPCODE_DEVICE_FOUND) {
    			msg.event = EVENT_UPDATE_STATE;
    			msg.state = APP_STATE_DEVICE_FOUND;
			}

            k_msgq_put(&app_msgq, &msg, K_NO_WAIT);
        }
    }
	else {
		// We are not paired with the current device yet, read the device ID and prepare for pairing.
		//
		if (opcode == OPCODE_GET_MOBILE_ID)
		{
			uint8_t *pu8_data = (uint8_t *)data;
			memcpy(current_dev_info.id, &pu8_data[BYTE_OFF_DATA], sizeof(current_dev_info.id));

			msg.event = EVENT_UPDATE_STATE;
			msg.state = APP_STATE_CHECK_EXISTENCE;
			k_msgq_put(&app_msgq, &msg, K_NO_WAIT);
		}
	}

	return BT_GATT_ITER_CONTINUE;
}

/**
 * @brief Discover callback function
 *
 * @param conn
 * @param attr
 * @param params
 * @return uint8_t
 */
static uint8_t discover_cb (struct bt_conn *conn, const struct bt_gatt_attr *attr,
			                struct bt_gatt_discover_params *params)
{
	int err;
	static app_task_msg_t msg;
	static struct bt_uuid_128 uuid = BT_UUID_INIT_128(0);

	if (!attr) {
		// The control should not come here. Just in case, the attribute handles are not handled
        // properly it comes, disconnect the peer and wait for reconnection
        //
		(void)memset(params, 0, sizeof(*params));
		bt_conn_disconnect(conn, BT_HCI_ERR_REMOTE_USER_TERM_CONN);

		return BT_GATT_ITER_STOP;
	}

	if ((!bt_uuid_cmp(g_discover_params.uuid, BT_UUID_NOTIFY_DEV)) ||
	    (!bt_uuid_cmp(g_discover_params.uuid, BT_UUID_MOBILE_DEV)))
    {
		// Found the NOTIF_DEV_SERVICE. Look for the CMD characteristic handle
        //
		memcpy(&uuid, BT_UUID_CMD, sizeof(uuid));
		g_discover_params.uuid = &uuid.uuid;
		g_discover_params.start_handle = attr->handle + 1;
		g_discover_params.type = BT_GATT_DISCOVER_CHARACTERISTIC;
		bt_gatt_discover(conn, &g_discover_params);
	}
    else if (!bt_uuid_cmp(g_discover_params.uuid, BT_UUID_CMD)) {
		// Found the NOTIF_DEV_CMD_CHR handle. Find the DESCRIPTOR handle
		//
		memcpy(&uuid, BT_UUID_GATT_CCC, sizeof(struct bt_uuid_16));
		g_discover_params.uuid = &uuid.uuid;
		g_discover_params.start_handle = attr->handle + 2;
		g_discover_params.type = BT_GATT_DISCOVER_DESCRIPTOR;
		g_subscribe_params.value_handle = bt_gatt_attr_value_handle(attr);

		// This is the only characteristic handle that we are going to use. So save it for
        // communication with the peripheral
        //
		current_dev_info.cmd_chr_handle = (attr->handle + 1);
		bt_gatt_discover(conn, &g_discover_params);
	}
    else {
		/* Subscribe to the INDICATIONS, this will enable indications
		   on the peripheral for the CMD characteristic
		 */
		g_subscribe_params.notify = indication_cb;
		g_subscribe_params.value = BT_GATT_CCC_INDICATE;
		g_subscribe_params.ccc_handle = attr->handle;
		err = bt_gatt_subscribe(conn, &g_subscribe_params);
		if (err == 0) {
            msg.event = EVENT_UPDATE_STATE;
			msg.state = APP_STATE_GET_DEVICE_ID;
			k_msgq_put(&app_msgq, &msg, K_NO_WAIT);
		}
	}

	return BT_GATT_ITER_STOP;
}

/**
 * @brief Extended inquiry response callback. This function is called multiple times by the bt_parse
 * function for each of the element found in the advertisement
 *
 * @param data
 * @param user_data
 * @return true
 * @return false
 */
static bool eir_found (struct bt_data *data, void *user_data)
{
    bool ret = 1;
	bt_addr_le_t *addr = user_data;
	char addr_str[BT_ADDR_LE_STR_LEN];
	int i;

    do {
    	if (data->type == BT_DATA_UUID128_SOME || data->type == BT_DATA_UUID128_ALL) {
    		if (data->data_len % sizeof(uint16_t) != 0U) {
                break;
    		}

    		bt_addr_le_to_str(addr, addr_str, sizeof(addr_str));

    		for (i = 0; i < data->data_len; i += 16) {
    			struct bt_le_conn_param *param;
    			int err;

    			g_peer_device_uuid.uuid.type = BT_UUID_TYPE_128;
    			memcpy(g_peer_device_uuid.val, &data->data[i], 16);

    			if ((bt_uuid_cmp((struct bt_uuid *)&g_peer_device_uuid, BT_UUID_NOTIFY_DEV)) &&
    				(bt_uuid_cmp((struct bt_uuid *)&g_peer_device_uuid, BT_UUID_MOBILE_DEV))) {
                    // Not the expecting service, iterate further
                    //
    				continue;
    			}

    			current_dev_info.is_mobile = !(bt_uuid_cmp((struct bt_uuid *)&g_peer_device_uuid,
                                               BT_UUID_MOBILE_DEV));
    			if (current_dev_info.is_mobile) {
    				memcpy(&g_peer_device_uuid, BT_UUID_NOTIFY_DEV, sizeof(struct bt_uuid_128));
    			}

    			err = stop_scan();
    			if (err) {
                    // Scan stop failed, return
                    //
                    break;
    			}

    			struct bt_conn *conn;
    			param = BT_LE_CONN_PARAM_DEFAULT;
    			err = bt_conn_le_create(addr, BT_CONN_LE_CREATE_CONN, param, &conn);
    			if (err) {
                    // Connection creation failed, start scan for other devices
                    //
    				start_scan();
                    break;
    			}

    			ret = 0; // success
    		}
        }
    } while (0);

	return ret;
}

/**
 * @brief Callback function called when the beacon device found with the defined UUID
 *
 * @param addr
 * @param rssi
 * @param type
 * @param ad
 */
static void device_found (const bt_addr_le_t *addr, int8_t rssi, uint8_t type,
			 			  struct net_buf_simple *ad)
{
	char dev[BT_ADDR_LE_STR_LEN];

	bt_addr_le_to_str(addr, dev, sizeof(dev));

	/* We're only interested in connectable events */
    g_mobile_id = 0; // fresh device is trying to connect
	bt_data_parse(ad, eir_found, (void *)addr);
}

/**
 * @brief Discover peer GATT services
 *
 */
void discover_services (void)
{
	g_discover_params.uuid = &g_peer_device_uuid.uuid;
	g_discover_params.func = discover_cb;
	g_discover_params.start_handle = BT_ATT_FIRST_ATTRIBUTE_HANDLE;
	g_discover_params.end_handle = BT_ATT_LAST_ATTRIBUTE_HANDLE;
	g_discover_params.type = BT_GATT_DISCOVER_PRIMARY;

	bt_gatt_discover(current_dev_info.conn, &g_discover_params);
}

/**
 * @brief Callback function for GATT write complete
 *
 * @param conn
 * @param err
 * @param params
 */
void write_resp_cb (struct bt_conn *conn, uint8_t err, struct bt_gatt_write_params *params)
{
	app_task_msg_t msg;

    g_dm_index = get_paired_device_slot_from_conn_param(conn);
	if (g_dm_index == -1) {
		return;
	}

    if (gb_dm_write_resp_wait) {
        gb_dm_write_resp_wait = false;

    	msg.event = EVENT_START_DM;
    	msg.data  = &g_dm_index;
    	k_msgq_put(&app_msgq, &msg, K_NO_WAIT);
    }
}

/**
 * @brief Send the requested data over BLE using GATT write
 *
 * @param dev_index - index of the beacon info in the paired list
 * @param data      - pointer to the data to send
 * @param len       - length of the data to send
 */
void app_gatt_write (int8_t dev_index, uint8_t *data, uint16_t len)
{
    if (paired_list[dev_index].conn_status == CONN_STATUS_DISCONNECTED ||
		paired_list[dev_index].conn == NULL) {
        return;
    }

	static struct bt_gatt_write_params params;
	params.handle = paired_list[dev_index].cmd_chr_handle;
	params.data   = data;
	params.length = len;
	params.func   = write_resp_cb;

	bt_gatt_write(paired_list[dev_index].conn, &params);
}

/**
 * @brief Start scanning for beacon advertisements
 *
 * @return int
 */
int start_scan (void)
{
    int err = -1;

    if (!gb_scanning_in_progress) {
    	struct bt_le_scan_param scan_param = {
    		.type     = BT_HCI_LE_SCAN_ACTIVE,
    		.options  = BT_LE_SCAN_OPT_NONE,
    		.interval = SCAN_INTERVAL,
    		.window   = SCAN_WINDOW,
            .timeout  = 0, // disable timeout
    	};

    	err = bt_le_scan_start(&scan_param, device_found);
        if (!err) {
            gb_scanning_in_progress = true;
        }
    }

	return 0;
}

/**
 * @brief Stop scanning for beacon advertisements
 *
 * @return int
 */
int
stop_scan (void)
{
	int ret = bt_le_scan_stop();
    if (ret == 0) {
        gb_scanning_in_progress = false;
    }

	return ret;
}

/**
 * @brief Send requested message to all the paired devices over GATT write
 *
 * @param p_msg_ctx
 */
void central_send_msg_to_all_paired_devices (message_context_t *p_msg_ctx)
{
    if (app_nvs.param.paired_device_count <= MAX_BEACONS_SUPPORTED) {
    	uint8_t data[20];

    	// Pack according BLE packet
    	//
    	data[BYTE_OFF_OPCODE] = p_msg_ctx->opcode;
    	data[BYTE_OFF_LEN]    = p_msg_ctx->len;
    	if (p_msg_ctx->len) {
    		memcpy(&data[BYTE_OFF_DATA], p_msg_ctx->data, p_msg_ctx->len);
    	}

    	// Send the message to all device one after the other
    	//
    	for (uint8_t index = 0; index < app_nvs.param.paired_device_count; index++) {
    		if (paired_list[index].conn_status == CONN_STATUS_CONNECTED &&
    			paired_list[index].pair_status)
    		{
    			app_gatt_write(index, data, (data[BYTE_OFF_LEN] + BLE_PKT_HEADER_SZ));
                k_sleep(K_MSEC(100));
    		}
    	}
    }
}

/**
 * @brief register all the necessary callbacks for the central application
 *
 * @return int
 */
void init_central (void)
{
	bt_conn_cb_register(&conn_callbacks);
}

/**
 * @brief Set the tx power object
 *
 * @param handle
 * @param tx_pwr_lvl
 */
void
set_tx_power (uint16_t handle, int8_t tx_pwr_lvl)
{
	struct bt_hci_cp_vs_write_tx_power_level *p_cp;
	struct bt_hci_rp_vs_write_tx_power_level *p_rp;
	struct net_buf *buf, *rsp = NULL;
	int err;

	buf = bt_hci_cmd_create(BT_HCI_OP_VS_WRITE_TX_POWER_LEVEL, sizeof(*p_cp));
	if (!buf) {
		return;
	}

	p_cp = net_buf_add(buf, sizeof(*p_cp));
	p_cp->handle = sys_cpu_to_le16(handle);
	p_cp->handle_type = BT_HCI_VS_LL_HANDLE_TYPE_CONN;
	p_cp->tx_power_level = tx_pwr_lvl;

	err = bt_hci_cmd_send_sync(BT_HCI_OP_VS_WRITE_TX_POWER_LEVEL, buf, &rsp);
	if (err) {
		return;
	}

	p_rp = (void *)rsp->data;

	net_buf_unref(rsp);
}

/**
 * @brief Read the tx power for the connected device
 *
 * @param handle
 * @param rssi
 */
void
read_conn_tx_power (uint16_t handle, int8_t *p_tx_pwr)
{
	struct bt_hci_cp_vs_read_tx_power_level *p_cp;
	struct bt_hci_rp_vs_read_tx_power_level *p_rp;
	struct net_buf *buf, *rsp = NULL;
	int err;

	*p_tx_pwr = 0xFF;
	buf = bt_hci_cmd_create(BT_HCI_OP_VS_READ_TX_POWER_LEVEL, sizeof(*p_cp));
	if (!buf) {
		return;
	}

	p_cp = net_buf_add(buf, sizeof(*p_cp));
	p_cp->handle = sys_cpu_to_le16(handle);
	p_cp->handle_type = BT_HCI_VS_LL_HANDLE_TYPE_CONN;

	err = bt_hci_cmd_send_sync(BT_HCI_OP_VS_READ_TX_POWER_LEVEL, buf, &rsp);
	if (err) {
		return;
	}

	p_rp = (void *)rsp->data;
	*p_tx_pwr = p_rp->tx_power_level;

	net_buf_unref(rsp);
}

/**
 * @brief Read the signal strength for the connected device
 *
 * @param handle
 * @param rssi
 */
void read_conn_rssi (uint16_t handle, int8_t *rssi)
{
	struct net_buf *buf, *rsp = NULL;
	struct bt_hci_cp_read_rssi *p_cp;
	struct bt_hci_rp_read_rssi *p_rp;

	int err;

	buf = bt_hci_cmd_create(BT_HCI_OP_READ_RSSI, sizeof(*p_cp));
	if (!buf) {
		return;
	}

	p_cp = net_buf_add(buf, sizeof(*p_cp));
	p_cp->handle = sys_cpu_to_le16(handle);

	err = bt_hci_cmd_send_sync(BT_HCI_OP_READ_RSSI, buf, &rsp);
	if (err) {
		return;
	}

	p_rp = (void *)rsp->data;
	*rssi = p_rp->rssi;

	net_buf_unref(rsp);
}

/**
 * @brief Get the paired dev slot from the paired list
 *
 * @param id - pointer to the peer unique 6 bytes ID
 * @return int8_t, index in the paired list only if already paired with.
 */
int8_t get_paired_device_slot_from_id_param (const uint8_t *id)
{
	int8_t slot = -1;

	for (int8_t i = 0; i < MAX_BEACONS_SUPPORTED; i++) {
        if ((!memcmp(paired_list[i].id, id, BT_ADDR_SIZE)) && (paired_list[i].pair_status)) {
			slot = i;
			break;
		}
	}

	return slot;
}

/**
 * @brief Remove paired info for the requested device from the paired list
 * Replace with the successive information list
 *
 * @param addr
 */
bool remove_paired_dev (uint8_t index)
{
    /* Disconnect with the peer if alive */
	int err;

    if (paired_list[index].conn) {
    	err = bt_conn_disconnect(paired_list[index].conn, BT_HCI_ERR_REMOTE_USER_TERM_CONN);
    	if (err) {
    	}
    }

    /* Move all the successive devices by 1 pre index */
    if (app_nvs.param.paired_device_count) {
    	for (; index < (app_nvs.param.paired_device_count - 1); index++) {
            paired_list[index] = paired_list[index + 1];
    	}
    }

    /* As we have copied all the devices to 1 pre index, the last device will have dual copy.
    Delete the last device info */
    memset(&paired_list[index], 0, sizeof(paired_dev_info_t));
    return true;
}

/**
 * @brief Get the paired dev slot from the paired list by comparing conn param
 *
 * @param p_conn: pointer to the device conn info
 * @return int8_t, index in the paired list only if already paired with.
 */
int8_t get_paired_device_slot_from_conn_param (struct bt_conn *p_conn)
{
	int8_t slot = -1;

	for (int8_t i = 0; i < MAX_BEACONS_SUPPORTED; i++) {
        if (paired_list[i].conn == p_conn) {
			slot = i;
			break;
		}
	}

	return slot;
}
